@@ -1,16 +1,19 @@
 """Ansible runtime environment manager."""
+import contextlib
 import importlib
 import json
 import logging
 import os
-import pathlib
 import re
 import shutil
 import subprocess
 import tempfile
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Union
+import warnings
+from collections import OrderedDict
+from dataclasses import dataclass
+from pathlib import Path
+from typing import TYPE_CHECKING, Any, Callable, Optional, Union
 
-import packaging
 import subprocess_tee
 from packaging.version import Version
 
@@ -19,7 +22,7 @@ from ansible_compat.config import (
     ansible_collections_path,
     parse_ansible_version,
 )
-from ansible_compat.constants import MSG_INVALID_FQRL
+from ansible_compat.constants import MSG_INVALID_FQRL, RC_ANSIBLE_OPTIONS_ERROR
 from ansible_compat.errors import (
     AnsibleCommandError,
     AnsibleCompatError,
@@ -41,24 +44,51 @@ _logger = logging.getLogger(__name__)
 version_re = re.compile(":[>=<]*([^,]*)")
 
 
+class AnsibleWarning(Warning):
+    """Warnings related to Ansible runtime."""
+
+
+@dataclass
+class Collection:
+    """Container for Ansible collection information."""
+
+    name: str
+    version: str
+    path: Path
+
+
+class CollectionVersion(Version):
+    """Collection version."""
+
+    def __init__(self, version: str) -> None:
+        """Initialize collection version."""
+        # As packaging Version class does not support wildcard, we convert it
+        # to "0", as this being the smallest version possible.
+        if version == "*":
+            version = "0"
+        super().__init__(version)
+
+
+# pylint: disable=too-many-instance-attributes
 class Runtime:
     """Ansible Runtime manager."""
 
-    _version: Optional[packaging.version.Version] = None
-    cache_dir: Optional[str] = None
+    _version: Optional[Version] = None
+    collections: OrderedDict[str, Collection] = OrderedDict()
+    cache_dir: Optional[Path] = None
     # Used to track if we have already initialized the Ansible runtime as attempts
     # to do it multiple tilmes will cause runtime warnings from within ansible-core
     initialized: bool = False
 
-    # pylint: disable=too-many-arguments
     def __init__(
         self,
-        project_dir: Optional[str] = None,
+        project_dir: Optional[Path] = None,
+        *,
         isolated: bool = False,
         min_required_version: Optional[str] = None,
         require_module: bool = False,
         max_retries: int = 0,
-        environ: Optional[Dict[str, str]] = None,
+        environ: Optional[dict[str, str]] = None,
     ) -> None:
         """Initialize Ansible runtime environment.
 
@@ -81,7 +111,7 @@ class Runtime:
         :param environ: Environment dictionary to use, if undefined
                         ``os.environ`` will be copied and used.
         """
-        self.project_dir = project_dir or os.getcwd()
+        self.project_dir = project_dir or Path.cwd()
         self.isolated = isolated
         self.max_retries = max_retries
         self.environ = environ or os.environ.copy()
@@ -99,36 +129,89 @@ class Runtime:
         self.config = AnsibleConfig()
 
         if not self.version_in_range(lower=min_required_version):
-            raise RuntimeError(
-                f"Found incompatible version of ansible runtime {self.version}, instead of {min_required_version} or newer."
-            )
+            msg = f"Found incompatible version of ansible runtime {self.version}, instead of {min_required_version} or newer."
+            raise RuntimeError(msg)
         if require_module:
             self._ensure_module_available()
 
+        # pylint: disable=import-outside-toplevel
+        from ansible.utils.display import Display
+
+        # pylint: disable=unused-argument
+        def warning(
+            self: Display,  # noqa: ARG001
+            msg: str,
+            *,
+            formatted: bool = False,  # noqa: ARG001
+        ) -> None:
+            """Override ansible.utils.display.Display.warning to avoid printing warnings."""
+            warnings.warn(
+                message=msg,
+                category=AnsibleWarning,
+                stacklevel=2,
+                source={"msg": msg},
+            )
+
+        # Monkey patch ansible warning in order to use warnings module.
+        Display.warning = warning
+
+    def load_collections(self) -> None:
+        """Load collection data."""
+        self.collections = OrderedDict()
+        no_collections_msg = "None of the provided paths were usable"
+
+        # Workaround for https://github.com/ansible/ansible/issues/73127
+        Path("~/.ansible/collections").expanduser().mkdir(exist_ok=True, parents=True)
+
+        proc = self.run(["ansible-galaxy", "collection", "list", "--format=json"])
+        if proc.returncode == RC_ANSIBLE_OPTIONS_ERROR and (
+            no_collections_msg in proc.stdout or no_collections_msg in proc.stderr
+        ):
+            _logger.debug("Ansible reported no installed collections at all.")
+            return
+        if proc.returncode != 0:
+            _logger.error(proc)
+            msg = f"Unable to list collections: {proc}"
+            raise RuntimeError(msg)
+        data = json.loads(proc.stdout)
+        if not isinstance(data, dict):
+            msg = f"Unexpected collection data, {data}"
+            raise TypeError(msg)
+        for path in data:
+            for collection, collection_info in data[path].items():
+                if not isinstance(collection, str):
+                    msg = f"Unexpected collection data, {collection}"
+                    raise TypeError(msg)
+                if not isinstance(collection_info, dict):
+                    msg = f"Unexpected collection data, {collection_info}"
+                    raise TypeError(msg)
+
+                self.collections[collection] = Collection(
+                    name=collection,
+                    version=collection_info["version"],
+                    path=path,
+                )
+
     def _ensure_module_available(self) -> None:
         """Assure that Ansible Python module is installed and matching CLI version."""
         ansible_release_module = None
-        try:
+        with contextlib.suppress(ModuleNotFoundError, ImportError):
             ansible_release_module = importlib.import_module("ansible.release")
-        except (ModuleNotFoundError, ImportError):
-            pass
 
         if ansible_release_module is None:
-            raise RuntimeError("Unable to find Ansible python module.")
+            msg = "Unable to find Ansible python module."
+            raise RuntimeError(msg)
 
-        ansible_module_version = packaging.version.parse(
-            ansible_release_module.__version__
+        ansible_module_version = Version(
+            ansible_release_module.__version__,
         )
         if ansible_module_version != self.version:
-            raise RuntimeError(
-                f"Ansible CLI ({self.version}) and python module"
-                f" ({ansible_module_version}) versions do not match. This "
-                "indicates a broken execution environment."
-            )
+            msg = f"Ansible CLI ({self.version}) and python module ({ansible_module_version}) versions do not match. This indicates a broken execution environment."
+            raise RuntimeError(msg)
 
         # For ansible 2.15+ we need to initialize the plugin loader
         # https://github.com/ansible/ansible-lint/issues/2945
-        if not Runtime.initialized:  # noqa: F823
+        if not Runtime.initialized:
             col_path = [f"{self.cache_dir}/collections"]
             if self.version >= Version("2.15.0.dev0"):
                 # pylint: disable=import-outside-toplevel,no-name-in-module
@@ -144,11 +227,11 @@ class Runtime:
                 # noinspection PyProtectedMember
                 # pylint: disable=protected-access
                 col_path += self.config.collections_paths
-                col_path += os.path.dirname(
-                    os.environ.get(ansible_collections_path(), ".")
+                col_path += os.path.dirname(  # noqa: PTH120
+                    os.environ.get(ansible_collections_path(), "."),
                 ).split(":")
-                _AnsibleCollectionFinder(
-                    paths=col_path
+                _AnsibleCollectionFinder(  # noqa: SLF001
+                    paths=col_path,
                 )._install()  # pylint: disable=protected-access
             Runtime.initialized = True
 
@@ -157,13 +240,14 @@ class Runtime:
         if self.cache_dir:
             shutil.rmtree(self.cache_dir, ignore_errors=True)
 
-    def exec(
+    def run(  # ruff: disable=PLR0913
         self,
-        args: Union[str, List[str]],
+        args: Union[str, list[str]],
+        *,
         retry: bool = False,
         tee: bool = False,
-        env: Optional[Dict[str, str]] = None,
-        cwd: Optional[str] = None,
+        env: Optional[dict[str, str]] = None,
+        cwd: Optional[Path] = None,
     ) -> CompletedProcess:
         """Execute a command inside an Ansible environment.
 
@@ -183,7 +267,7 @@ class Runtime:
                 stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE,
                 env=env or self.environ,
-                cwd=cwd,
+                cwd=str(cwd) if cwd else None,
             )
             if result.returncode == 0:
                 break
@@ -196,7 +280,7 @@ class Runtime:
         return result
 
     @property
-    def version(self) -> packaging.version.Version:
+    def version(self) -> Version:
         """Return current Version object for Ansible.
 
         If version is not mentioned, it returns current version as detected.
@@ -206,7 +290,7 @@ class Runtime:
         if self._version:
             return self._version
 
-        proc = self.exec(["ansible", "--version"])
+        proc = self.run(["ansible", "--version"])
         if proc.returncode == 0:
             self._version = parse_ansible_version(proc.stdout)
             return self._version
@@ -215,22 +299,25 @@ class Runtime:
         raise MissingAnsibleError(msg, proc=proc)
 
     def version_in_range(
-        self, lower: Optional[str] = None, upper: Optional[str] = None
+        self,
+        lower: Optional[str] = None,
+        upper: Optional[str] = None,
     ) -> bool:
         """Check if Ansible version is inside a required range.
 
         The lower limit is inclusive and the upper one exclusive.
         """
-        if lower and self.version < packaging.version.Version(lower):
+        if lower and self.version < Version(lower):
             return False
-        if upper and self.version >= packaging.version.Version(upper):
+        if upper and self.version >= Version(upper):
             return False
         return True
 
     def install_collection(
         self,
-        collection: str,
-        destination: Optional[Union[str, pathlib.Path]] = None,
+        collection: Union[str, Path],
+        *,
+        destination: Optional[Path] = None,
         force: bool = False,
     ) -> None:
         """Install an Ansible collection.
@@ -249,11 +336,11 @@ class Runtime:
         # As ansible-galaxy install is not able to automatically determine
         # if the range requires a pre-release, we need to manuall add the --pre
         # flag when needed.
-        matches = version_re.search(collection)
-        if matches and Version(matches[1]).is_prerelease:
+        matches = version_re.search(str(collection))
+        if matches and CollectionVersion(matches[1]).is_prerelease:
             cmd.append("--pre")
 
-        cpaths: List[str] = self.config.collections_paths
+        cpaths: list[str] = self.config.collections_paths
         if destination and str(destination) not in cpaths:
             # we cannot use '-p' because it breaks galaxy ability to ignore already installed collections, so
             # we hack ansible_collections_path instead and inject our own path there.
@@ -261,8 +348,8 @@ class Runtime:
             cpaths.insert(0, str(destination))
         cmd.append(f"{collection}")
 
-        _logger.info("Running from %s : %s", os.getcwd(), " ".join(cmd))
-        run = self.exec(
+        _logger.info("Running from %s : %s", Path.cwd(), " ".join(cmd))
+        run = self.run(
             cmd,
             retry=True,
             env={**self.environ, ansible_collections_path(): ":".join(cpaths)},
@@ -273,7 +360,9 @@ class Runtime:
             raise InvalidPrerequisiteError(msg)
 
     def install_collection_from_disk(
-        self, path: str, destination: Optional[Union[str, pathlib.Path]] = None
+        self,
+        path: Path,
+        destination: Optional[Path] = None,
     ) -> None:
         """Build and install collection from a given disk path."""
         if not self.version_in_range(upper="2.11"):
@@ -286,24 +375,28 @@ class Runtime:
                 "collection",
                 "build",
                 "--output-path",
-                tmp_dir,
-                path,
+                str(tmp_dir),
+                str(path),
             ]
             _logger.info("Running %s", " ".join(cmd))
-            run = self.exec(cmd, retry=False)
+            run = self.run(cmd, retry=False)
             if run.returncode != 0:
                 _logger.error(run.stdout)
                 raise AnsibleCommandError(run)
             for archive_file in os.listdir(tmp_dir):
                 self.install_collection(
-                    os.path.join(tmp_dir, archive_file),
+                    str(Path(tmp_dir) / archive_file),
                     destination=destination,
                     force=True,
                 )
 
     # pylint: disable=too-many-branches
-    def install_requirements(
-        self, requirement: str, retry: bool = False, offline: bool = False
+    def install_requirements(  # noqa: C901
+        self,
+        requirement: Path,
+        *,
+        retry: bool = False,
+        offline: bool = False,
     ) -> None:
         """Install dependencies from a requirements.yml.
 
@@ -311,13 +404,12 @@ class Runtime:
         :param retry: retry network operations on failures
         :param offline: bypass installation, may fail if requirements are not met.
         """
-        if not os.path.exists(requirement):
+        if not Path(requirement).exists():
             return
-        reqs_yaml = yaml_from_file(requirement)
+        reqs_yaml = yaml_from_file(Path(requirement))
         if not isinstance(reqs_yaml, (dict, list)):
-            raise InvalidPrerequisiteError(
-                f"{requirement} file is not a valid Ansible requirements file."
-            )
+            msg = f"{requirement} file is not a valid Ansible requirements file."
+            raise InvalidPrerequisiteError(msg)
 
         if isinstance(reqs_yaml, list) or "roles" in reqs_yaml:
             cmd = [
@@ -332,12 +424,12 @@ class Runtime:
 
             if offline:
                 _logger.warning(
-                    "Skipped installing old role dependencies due to running in offline mode."
+                    "Skipped installing old role dependencies due to running in offline mode.",
                 )
             else:
                 _logger.info("Running %s", " ".join(cmd))
 
-                result = self.exec(cmd, retry=retry)
+                result = self.run(cmd, retry=retry)
                 if result.returncode != 0:
                     _logger.error(result.stdout)
                     raise AnsibleCommandError(result)
@@ -352,10 +444,10 @@ class Runtime:
             ]
             if offline:
                 _logger.warning(
-                    "Skipped installing collection dependencies due to running in offline mode."
+                    "Skipped installing collection dependencies due to running in offline mode.",
                 )
             else:
-                cmd.extend(["-r", requirement])
+                cmd.extend(["-r", str(requirement)])
                 cpaths = self.config.collections_paths
                 if self.cache_dir:
                     # we cannot use '-p' because it breaks galaxy ability to ignore already installed collections, so
@@ -365,7 +457,7 @@ class Runtime:
                         # pylint: disable=no-member
                         cpaths.insert(0, dest_path)
                 _logger.info("Running %s", " ".join(cmd))
-                result = self.exec(
+                result = self.run(
                     cmd,
                     retry=retry,
                     env={**os.environ, "ANSIBLE_COLLECTIONS_PATH": ":".join(cpaths)},
@@ -377,13 +469,15 @@ class Runtime:
 
     def prepare_environment(  # noqa: C901
         self,
-        required_collections: Optional[Dict[str, str]] = None,
+        required_collections: Optional[dict[str, str]] = None,
+        *,
         retry: bool = False,
         install_local: bool = False,
         offline: bool = False,
         role_name_check: int = 0,
     ) -> None:
         """Make dependencies available if needed."""
+        destination: Optional[Path] = None
         if required_collections is None:
             required_collections = {}
 
@@ -395,10 +489,28 @@ class Runtime:
             "requirements.yml",
             "roles/requirements.yml",
             "collections/requirements.yml",
+            # These is more of less the official way to store test requirements in collections so far:
+            "tests/requirements.yml",
         ]:
-            self.install_requirements(req_file, retry=retry, offline=offline)
+            self.install_requirements(Path(req_file), retry=retry, offline=offline)
 
-        destination = f"{self.cache_dir}/collections" if self.cache_dir else None
+        galaxy_path = Path("galaxy.yml")
+        if galaxy_path.exists():
+            data = yaml_from_file(galaxy_path)
+            if isinstance(data, dict) and "dependencies" in data:
+                for name, required_version in data["dependencies"].items():
+                    _logger.info(
+                        "Provisioning collection %s:%s from galaxy.yml",
+                        name,
+                        required_version,
+                    )
+                    self.install_collection(
+                        f"{name}:{required_version}",
+                        destination=destination,
+                    )
+
+        if self.cache_dir:
+            destination = self.cache_dir / "collections"
         for name, min_version in required_collections.items():
             self.install_collection(
                 f"{name}:>={min_version}",
@@ -410,40 +522,48 @@ class Runtime:
         if not install_local:
             return
 
-        if os.path.exists("galaxy.yml"):
+        if Path("galaxy.yml").exists():
             if destination:
                 # while function can return None, that would not break the logic
-                colpath = f"{destination}/ansible_collections/{colpath_from_path(os.getcwd())}"
-                if os.path.islink(colpath):
-                    if os.path.realpath(colpath) == os.getcwd():
+                colpath = Path(
+                    f"{destination}/ansible_collections/{colpath_from_path(Path.cwd())}",
+                )
+                if colpath.is_symlink():
+                    if os.path.realpath(colpath) == Path.cwd():
                         _logger.warning(
-                            "Found symlinked collection, skipping its installation."
+                            "Found symlinked collection, skipping its installation.",
                         )
                         return
                     _logger.warning(
                         "Collection is symlinked, but not pointing to %s directory, so we will remove it.",
-                        os.getcwd(),
+                        Path.cwd(),
                     )
-                    os.unlink(colpath)
+                    colpath.unlink()
 
             # molecule scenario within a collection
-            self.install_collection_from_disk(".", destination=destination)
-        elif pathlib.Path().resolve().parent.name == "roles" and os.path.exists(
-            "../../galaxy.yml"
+            self.install_collection_from_disk(Path("."), destination=destination)
+        elif (
+            Path().resolve().parent.name == "roles"
+            and Path("../../galaxy.yml").exists()
         ):
             # molecule scenario located within roles/<role-name>/molecule inside
             # a collection
-            self.install_collection_from_disk("../..", destination=destination)
+            self.install_collection_from_disk(Path("../.."), destination=destination)
         else:
             # no collection, try to recognize and install a standalone role
             self._install_galaxy_role(
-                self.project_dir, role_name_check=role_name_check, ignore_errors=True
+                self.project_dir,
+                role_name_check=role_name_check,
+                ignore_errors=True,
             )
+        # reload collections
+        self.load_collections()
 
-    def require_collection(  # noqa: C901
+    def require_collection(
         self,
         name: str,
         version: Optional[str] = None,
+        *,
         install: bool = True,
     ) -> None:
         """Check if a minimal collection version is present or exits.
@@ -454,14 +574,16 @@ class Runtime:
         try:
             ns, coll = name.split(".", 1)
         except ValueError as exc:
+            msg = f"Invalid collection name supplied: {name}%s"
             raise InvalidPrerequisiteError(
-                f"Invalid collection name supplied: {name}%s"
+                msg,
             ) from exc
 
-        paths: List[str] = self.config.collections_paths
+        paths: list[str] = self.config.collections_paths
         if not paths or not isinstance(paths, list):
+            msg = f"Unable to determine ansible collection paths. ({paths})"
             raise InvalidPrerequisiteError(
-                f"Unable to determine ansible collection paths. ({paths})"
+                msg,
             )
 
         if self.cache_dir:
@@ -471,22 +593,20 @@ class Runtime:
             paths.insert(0, f"{self.cache_dir}/collections")  # pylint: disable=E1101
 
         for path in paths:
-            collpath = os.path.expanduser(
-                os.path.join(path, "ansible_collections", ns, coll)
-            )
-            if os.path.exists(collpath):
-                mpath = os.path.join(collpath, "MANIFEST.json")
-                if not os.path.exists(mpath):
+            collpath = Path(path) / "ansible_collections" / ns / coll
+            if collpath.exists():
+                mpath = collpath / "MANIFEST.json"
+                if not mpath.exists():
                     msg = f"Found collection at '{collpath}' but missing MANIFEST.json, cannot get info."
                     _logger.fatal(msg)
                     raise InvalidPrerequisiteError(msg)
 
-                with open(mpath, "r", encoding="utf-8") as f:
+                with mpath.open(encoding="utf-8") as f:
                     manifest = json.loads(f.read())
-                    found_version = packaging.version.parse(
-                        manifest["collection_info"]["version"]
+                    found_version = CollectionVersion(
+                        manifest["collection_info"]["version"],
                     )
-                    if version and found_version < packaging.version.parse(version):
+                    if version and found_version < CollectionVersion(version):
                         if install:
                             self.install_collection(f"{name}:>={version}")
                             self.require_collection(name, version, install=False)
@@ -507,11 +627,12 @@ class Runtime:
     def _prepare_ansible_paths(self) -> None:
         """Configure Ansible environment variables."""
         try:
-            library_paths: List[str] = self.config.default_module_path.copy()
-            roles_path: List[str] = self.config.default_roles_path.copy()
-            collections_path: List[str] = self.config.collections_paths.copy()
+            library_paths: list[str] = self.config.default_module_path.copy()
+            roles_path: list[str] = self.config.default_roles_path.copy()
+            collections_path: list[str] = self.config.collections_paths.copy()
         except AttributeError as exc:
-            raise RuntimeError("Unexpected ansible configuration") from exc
+            msg = "Unexpected ansible configuration"
+            raise RuntimeError(msg) from exc
 
         alterations_list = [
             (library_paths, "plugins/modules", True),
@@ -525,16 +646,17 @@ class Runtime:
                 (collections_path, f"{self.cache_dir}/collections", False),
             ]
             if self.isolated
-            else []
+            else [],
         )
 
-        for path_list, path, must_be_present in alterations_list:
-            if not os.path.exists(path):
+        for path_list, path_, must_be_present in alterations_list:
+            path = Path(path_)
+            if not path.exists():
                 if must_be_present:
                     continue
-                os.makedirs(path, exist_ok=True)
+                path.mkdir(parents=True, exist_ok=True)
             if path not in path_list:
-                path_list.insert(0, path)
+                path_list.insert(0, str(path))
 
         if library_paths != self.config.DEFAULT_MODULE_PATH:
             self._update_env("ANSIBLE_LIBRARY", library_paths)
@@ -543,7 +665,7 @@ class Runtime:
         if roles_path != self.config.default_roles_path:
             self._update_env("ANSIBLE_ROLES_PATH", roles_path)
 
-    def _get_roles_path(self) -> pathlib.Path:
+    def _get_roles_path(self) -> Path:
         """Return roles installation path.
 
         If `self.isolated` is set to `True`, `self.cache_dir` would be
@@ -552,13 +674,17 @@ class Runtime:
         `default_roles_path`.
         """
         if self.cache_dir:
-            path = pathlib.Path(f"{self.cache_dir}/roles")
+            path = Path(f"{self.cache_dir}/roles")
         else:
-            path = pathlib.Path(os.path.expanduser(self.config.default_roles_path[0]))
+            path = Path(self.config.default_roles_path[0]).expanduser()
         return path
 
     def _install_galaxy_role(
-        self, project_dir: str, role_name_check: int = 0, ignore_errors: bool = False
+        self,
+        project_dir: Path,
+        role_name_check: int = 0,
+        *,
+        ignore_errors: bool = False,
     ) -> None:
         """Detect standalone galaxy role and installs it.
 
@@ -576,9 +702,9 @@ class Runtime:
         """
         yaml = None
         galaxy_info = {}
-        meta_filename = os.path.join(project_dir, "meta", "main.yml")
+        meta_filename = Path(project_dir) / "meta" / "main.yml"
 
-        if not os.path.exists(meta_filename):
+        if not meta_filename.exists():
             if ignore_errors:
                 return
         else:
@@ -597,22 +723,21 @@ class Runtime:
                 else:
                     _logger.error(msg)
                     raise InvalidPrerequisiteError(msg)
-        else:
+        elif "role_name" in galaxy_info:
             # when 'role-name' is in skip_list, we stick to plain role names
-            if "role_name" in galaxy_info:
-                role_namespace = _get_galaxy_role_ns(galaxy_info)
-                role_name = _get_galaxy_role_name(galaxy_info)
-                fqrn = f"{role_namespace}{role_name}"
-            else:
-                fqrn = pathlib.Path(project_dir).absolute().name
+            role_namespace = _get_galaxy_role_ns(galaxy_info)
+            role_name = _get_galaxy_role_name(galaxy_info)
+            fqrn = f"{role_namespace}{role_name}"
+        else:
+            fqrn = Path(project_dir).absolute().name
         path = self._get_roles_path()
         path.mkdir(parents=True, exist_ok=True)
         link_path = path / fqrn
         # despite documentation stating that is_file() reports true for symlinks,
         # it appears that is_dir() reports true instead, so we rely on exists().
-        target = pathlib.Path(project_dir).absolute()
+        target = Path(project_dir).absolute()
         if not link_path.exists() or (
-            link_path.is_symlink() and os.readlink(link_path) != str(target)
+            link_path.is_symlink() and link_path.readlink() != target
         ):
             # must call unlink before checking exists because a broken
             # link reports as not existing and we want to repair it
@@ -624,7 +749,7 @@ class Runtime:
             link_path,
         )
 
-    def _update_env(self, varname: str, value: List[str], default: str = "") -> None:
+    def _update_env(self, varname: str, value: list[str], default: str = "") -> None:
         """Update colon based environment variable if needed.
 
         New values are prepended to make sure they take precedence.
@@ -640,27 +765,29 @@ class Runtime:
             _logger.info("Set %s=%s", varname, value_str)
 
 
-def _get_role_fqrn(galaxy_infos: Dict[str, Any], project_dir: str) -> str:
+def _get_role_fqrn(galaxy_infos: dict[str, Any], project_dir: Path) -> str:
     """Compute role fqrn."""
     role_namespace = _get_galaxy_role_ns(galaxy_infos)
     role_name = _get_galaxy_role_name(galaxy_infos)
 
     if len(role_name) == 0:
-        role_name = pathlib.Path(project_dir).absolute().name
+        role_name = Path(project_dir).absolute().name
         role_name = re.sub(r"(ansible-|ansible-role-)", "", role_name).split(
-            ".", maxsplit=2
+            ".",
+            maxsplit=2,
         )[-1]
 
     return f"{role_namespace}{role_name}"
 
 
-def _get_galaxy_role_ns(galaxy_infos: Dict[str, Any]) -> str:
+def _get_galaxy_role_ns(galaxy_infos: dict[str, Any]) -> str:
     """Compute role namespace from meta/main.yml, including trailing dot."""
     role_namespace = galaxy_infos.get("namespace", "")
     if len(role_namespace) == 0:
         role_namespace = galaxy_infos.get("author", "")
     if not isinstance(role_namespace, str):
-        raise AnsibleCompatError(f"Role namespace must be string, not {role_namespace}")
+        msg = f"Role namespace must be string, not {role_namespace}"
+        raise AnsibleCompatError(msg)
     # if there's a space in the name space, it's likely author name
     # and not the galaxy login, so act as if there was no namespace
     if not role_namespace or re.match(r"^\w+ \w+", role_namespace):
@@ -670,6 +797,6 @@ def _get_galaxy_role_ns(galaxy_infos: Dict[str, Any]) -> str:
     return role_namespace
 
 
-def _get_galaxy_role_name(galaxy_infos: Dict[str, Any]) -> str:
+def _get_galaxy_role_name(galaxy_infos: dict[str, Any]) -> str:
     """Compute role name from meta/main.yml."""
     return galaxy_infos.get("role_name", "")
